##################################################
# $Id: FunctionSelector.py,v 1.4 2002/11/12 22:22:25 jonen Exp $
# small plugin for glimmer
#
# Authors:
#  Andreas Motl<andreas.motl@ilo.de>
#  Sebastian Utz<su@tunemedia.de>
#
# ToDo:
# - Bug: FunctionList doesn't refresh on new opened files by its own
# - tune regex for more languages
#
# Changelog:
#  $Log: FunctionSelector.py,v $
#  Revision 1.4  2002/11/12 22:22:25  jonen
#  + added support for concurrent pygtk installations
#
#  Revision 1.3  2002/11/12 20:14:48  jonen
#  + php functions maybe prefixed by a '&'. fixed sigpart regex.
#  + forgot 'class' (php) after all. fixed regex.
#  + renamed regex_perl to regex_noparenthesis
#  + perl subs with an space after the first '{' weren't regognized. fixed regex.
#
#  Revision 1.2  2002/11/08 02:37:25  jonen
#  + added cvs header
#  + added hook for refresh on "save-file"
#    suggested by Mark McAulay
#
#
##################################################

import glimmer
import string
import re
import sys
import os
sys.argv=[]
sys.argv.append("null")
try:
   import pygtk
   pygtk.require("1.2")
except:
   pass
from gtk import *
import _gtk
import _gnome
import _gnomeui
from gnome.ui import *
import gnome.config

global row_number

row_number = []

# ---------------------------------------------------------
# some regular expressions to scan for 
# class/function/sub/packages signatures
# in source code
prefix_pattern = '^[|\s]*'
sigpart = '\s+([\w|:|&]+?)\s*'
regex_universal=re.compile(prefix_pattern + '(?:\w+?)\*??' + sigpart + '\(.*?\)[\s|:|{]*\s*$')
regex_noparenthesis=re.compile(prefix_pattern + '(?:package|sub|class)' + sigpart + '[;|{]\s*$')

def get_functionNameFromLine(str):
  for regex in (regex_universal, regex_noparenthesis):
    match=regex.search(str)
    if match:
      str = match.group(1)
      return str

def get_allFunctionNames(content):
  names=[]
  lines=string.split(content, "\n")
  count=1
  for line in lines:
    funcname=get_functionNameFromLine(line)
    if funcname:
      names.append((funcname, count))
    count=count+1
  return names

# ---------------------------------------------------------

def get_current_text():
   start = 0
   end = glimmer.buffer_size()
   str = glimmer.get_text(start, end)
   return str

def functionlist_init ():
   title = "Functions:"
   functionlist.set_column_title(0, title)
   functionlist.clear()

def functionlist_append(label):
   listentry=[]
   listentry.append(label)
   listentry.append("")
   functionlist.append(listentry)

def our_main():
  global row_number
  gettext = get_current_text()
  if gettext:  
    entries=get_allFunctionNames(gettext)
    functionlist_init()
    row_number=[]
    for entry in entries:
      label=entry[0]
      rownum=entry[1]
      functionlist_append(label)
      row_number.append(rownum)

def select_row_callback(clist, row, col, event, data):
   import glimmer
   import string
   global row_number
   num = row_number[row]
   glimmer.move_to_line(num)
   glimmer.move_to(glimmer.line_start())
   del glimmer
   del string


def close_function_selector(button, citem, fbox):
   citem.destroy()
   glimmer.remove_paned_object("FunctionSelector")


def refresh_function_selector(refresh):
   our_main()

def callback_change():
   our_main()

functionbox = GtkVBox(FALSE, 0)
functionbox.show()
titleframe = GtkFrame()
functionbox.pack_start(titleframe, FALSE, FALSE, 2)
titleframe.show()
buttonbox = GtkHBox(FALSE, 0)
titleframe.add(buttonbox)
buttonbox.show()
optionsbutton = GtkButton()
optionsbutton.set_relief(RELIEF_NONE)
buttonbox.pack_start(optionsbutton, FALSE, FALSE, 2)
optionsbutton.show()
optionsbuttonbox = GtkHBox(FALSE, 0)
optionsbutton.add(optionsbuttonbox)
optionsbuttonbox.pack_start(GtkLabel("Refresh List"), FALSE, FALSE, 0)
optionsbuttonbox.show_all()

_gtk.gtk_signal_connect(optionsbutton._o, "clicked", refresh_function_selector)
closebutton = GtkButton()
closebutton.set_relief(RELIEF_NONE)
closebutton.add(GnomeStock("Close"))
closebutton.set_usize(22, 22)
buttonbox.pack_end(closebutton, FALSE, FALSE, 2)
closebutton.show_all()
functionwindow = GtkScrolledWindow()
functionbox.pack_start(functionwindow, TRUE, TRUE, 2)
functionwindow.set_usize(130, 500)
functionwindow.show()
functionlist = GtkCList(1)
functionwindow.add(functionlist)
functionlist.show()
#functionlist.column_titles_show(1)
glimmer.add_paned_object(functionbox._o, "FunctionSelector", 0)
our_main()
checkitem = GtkCheckMenuItem("Function Selector")
checkitem.set_active(TRUE)
checkitem.show()
glimmer.add_widget_to_menu("_View/Status Bar", checkitem._o)

glimmer.register_signal_hook("open-file", callback_change)

glimmer.register_signal_hook("change-file", callback_change)

glimmer.register_signal_hook("save-file", callback_change)

args = (checkitem, functionbox, )
_gtk.gtk_signal_connect(closebutton._o, "clicked", close_function_selector, args)
args = (None, )
_gtk.gtk_signal_connect(functionlist._o, "select_row", select_row_callback, args)
